"""The package that contains all the modules used by the PhysBiblio,
including the `gui` subpackage.

This file is part of the physbiblio package.
"""
__author__ = 'Stefano Gariazzo'
__email__ = "stefano.gariazzo@gmail.com"

__version__ = "1.0.0"
__version_date__ = "18/02/2019"

__all__ = [
	"bibtexWriter",
	"cli",
	"config",
	"databaseCore",
	"database",
	"errors",
	"export",
	"inspireStats",
	"parseAccents",
	"pdf",
	"tablesDef",
	"view",
	"tests",
	"gui",
	"webimport",
	]
