"""The package that contains all the GUI modules used by the PhysBiblio.

This file is part of the physbiblio package.
"""
__author__ = 'Stefano Gariazzo'
__email__ = "stefano.gariazzo@gmail.com"

import physbiblio
__version__ = physbiblio.__version__
__version_date__ = physbiblio.__version_date__

__all__ = [
	"basicDialogs",
	"bibWindows",
	"catWindows",
	"commonClasses",
	"dialogWindows",
	"errorManager",
	"expWindows",
	"inspireStatsGUI",
	"mainWindow",
	"marks",
	"profilesManager",
	"resourcesPyside2",
	"setuptests",
	"threadElements",
	]
